<?php

namespace App\Models;

// use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Sanctum\HasApiTokens;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'name',
        'email',
        'password',
    ];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
        'password' => 'hashed',
    ];



    public function roles()
    {
        return $this->belongsToMany(Role::class)->withTimestamps(); //->with("permisos");
    }

    public function assignRole($role)
    {
        // "admin",
        // {nombre: admin, detalle: 'administrar todo'}
        if (is_string($role)) {
            $role = Role::where("nombre", $role)->firstOrFail();
        }

        $this->roles()->sync($role, false); //qiutamos el detach
    }

    public function removeRole($role)
    {
        // "admin",
        // {nombre: admin, detalle: 'administrar todo'} //puede llegar como onbjeto
        if (is_string($role)) {
            $role = Role::where("nombre", $role)->firstOrFail();
        }

        $this->roles()->detach($role);
    }

    public function isAdministrador()
    {
        return $this->roles->flatten()->pluck("nombre")->contain("admin");
    }
    //Funcion para pobtener todos los permisos en general
    public function permisos()
    {
        return $this->roles->map->permisos->flatten()->pluck('detalle')->unique();
    }


    public function client()
    {
        return $this->hasOne(Client::class);
    }
    public function facturas()
    {
        return $this->belongsToMany(Factura::class)->withTimestamps();
    }
}
